{{ RoBo_Remote_v9.spin

        RoBo Remote - Multi Robot Remote Controller

        TheGrue
        John Harris
        http://letsmakerobots.com/node/18692

        RoBo Remote - Check ID Pins and Acquire pulses for left and right motors to send to remote robot
}}

CON

  _xinfreq = 5_000_000          ' 5 MHz cyrstal
  _clkmode = xtal1 + pll16x     ' use crystal x 16 (sys clock = 80 MHz)
  
'' ---Pin Assignments as indicated by the "p_" beginning ---

  '-- XBee Transceiver --
  p_XBTx        = 14            ' XBee Digital Input DIN
  p_XBRx        = 15            ' XBee Digital Output DOUT

  '-- Accelerometer --
  p_CLKPIN      = 8             ' Accelerometer Clock Pin
  p_DATAPIN     = 9             ' Accelerometer Data Pin
  p_CSPIN       = 10            ' Accelerometer CS Pin

  '-- LED's --
  p_Power       = 0             ' LED
  p_Blue        = 1             ' LED
  p_Green       = 2             ' LED
  p_Yellow      = 3             ' LED
  p_Red         = 4             ' LED

  '-- XBee ID Switchs --
  p_IdLow       = 16            ' XBee ID Slider Switch
  p_IdMid       = 17            ' XBee ID Slider Switch
  p_IdHigh      = 18            ' XBee ID Slider Switch

  '-- Slide Switches --
  p_SldRight    = 19            ' Right Slider Switch
  p_SldMiddle   = 20            ' Center Slider Switch
  p_SldLeft     = 21            ' Left Slider Switch

  '-- Momentary Switches --
  p_BlkLeft     = 22            ' Left Black Momentary Switch
  p_BlkRight    = 23            ' Right Black Momentary Switch
  p_RedLeft     = 24            ' Left Red Momentary Switch
  p_RedRight    = 25            ' Right Red Momentary Switch

'' --- General Constants ---

  '-- XBee --
  XB_Baud       = 9600          ' XBee Baud Rate
  MY_Addr       = 0             ' XBee MY Address of the RoBo Remote


OBJ

  XB:           "XBee_Object"                           'XBee Communication Object
  Accel:        "Acc_2_Servo_v6"                        'Accelerometer Object
  vp:           "Conduit"                               'Viewport Debugger Object

VAR

  long LeftPulse, RightPulse, PulseMin, PulseMax, TurnAdjust, dataIn, DL_Addr, y
  byte  Stack[100]

PUB Main | Count

' --- Set Pins as Inputs or Outputs ---
  dira[p_Power..p_Red]~~
  dira[p_IdLow..p_RedRight]~
  dira[p_XBTx]~~
  dira[p_XBRx]~

'-------------- Initialization --------------
  outa[p_Power]~~                                       ' Ligh Red Power Switch LED
'     Diagnostics                                       ' Set ViewPort up to connect for Diagnostics - comment out when not used

  '-- LED Diagnosic Pattern --
   Repeat 4
    Repeat Count from 4 to 1
      Outa[Count]~~
      !outa[Count + 1]
      !outa[p_Power]
      waitcnt(clkfreq/12 + cnt)
    Repeat Count from 1 to 4
      Outa[Count]~~
      !outa[Count - 1]
      !outa[p_Power]
      waitcnt(clkfreq/12 + cnt)
  Outa[p_Red..p_Blue]~    
      
  '-- Initialize Accelerometer and variables --
  Accel.Init(p_CLKPIN, p_DATAPIN, p_CSPIN)
  LeftPulse  := 1500
  RightPulse := 1500
  PulseMin   := 1200            ' PulseMin = 1000-1400 Range - 1000 is full speed reverse and 1400 is slow reverse
  PulseMax   := 1750            ' PulseMax = 1600-2000 Range - 2000 is full speed forward and 1600 is slow forward
  TurnAdjust := 20              ' TurnAdjust = 5-50 - 5 is slow wide turns and 50 is sharp fast turns
   
  '-- Configure XBee --
  DL_Addr := ina[p_IdHigh..p_IdLow]                     ' Read the RoBo Remote ID pins and assign them to DL_Adder
  XB.start(p_XBRx, p_XBTx, 0, XB_Baud)                  ' Initialize comms for XBee
  XB.AT_Init                                            ' Fast AT updates
  XB.AT_ConfigVal(string("ATMY"), MY_Addr)              ' Assign RoBo Remote's ID address
  XB.AT_ConfigVal(string("ATDL"), DL_Addr)              ' Assign Destination ID address
' cognew(acceptData,@Stack)                             ' start cog to accept incoming data
   
'-------------- Main Routine --------------
  SendControl
     
Pub SendControl
  repeat
    if DL_Addr <> ina[p_IdHigh..p_IdLow]                ' If ID switches have been changed
        ChangeTransmitAddress                           ' Change the Destination ID address
      
    elseif ina[p_SldRight] == 0                         ' if Right Slider Switch off
        XB.str(string("h"))           '                 ' send handful of p's for pan map
        XB.delay(500)
         
    else
        ' Read Accelerometer for Left and Right pulses
        If ina[p_SldMiddle] == 0                        ' Middle slider switch is on, run at top speed
          PulseMin := 1200                              ' PulseMin = 1000-1400 Range - 1000 is full speed reverse and 1400 is slow reverse
          PulseMax := 1750                              ' PulseMax = 1600-2000 Range - 2000 is full speed forward and 1600 is slow forward     
'         Accel.ReadData(@LeftPulse, @RightPulse, PulseMin, PulseMax, TurnAdjust)

        else                                            ' Else if middle slider is off, run at half speed
          PulseMin := 1000                              ' PulseMin = 1000-1400 Range - 1000 is full speed reverse and 1400 is slow reverse
          PulseMax := 2000                              ' PulseMax = 1600-2000 Range - 2000 is full speed forward and 1600 is slow forward

        y := Accel.ReadData(@LeftPulse, @RightPulse, PulseMin, PulseMax, TurnAdjust)
         
        ' send drive data(d) to bot
        outa[p_Blue]~~
        XB.tx("d")
        XB.DEC(RightPulse)
        XB.tx(13)
        XB.DEC(LeftPulse)
        XB.tx(13)
        outa[p_Blue]~
         
    XB.delay(100)


Pub AcceptData
  '' accepts incoming data from bot to light Demo board LED's
  '' based on PING distance
  
'   dira[16..23]~~                     ' set LEDs as outputs
'   repeat
'     DataIn := XB.Rx                  ' read incoming data
'     if DataIn == "u" or DataIn == "m"  ' if update (u)or mapping (m)...
'       Range := XB.RxDecTime(100)     ' get range
'       if Range < 800                                                    
          ' if less the 0.8 meters (1 LED per 0.1m or 100mm)
          ' Light LEDs by shifting 1's  by scaled range
'         outa[16..23] := %11111111 << 8  >> (8 - range/100)
'       else                           ' if > 0.8m, turn off LEDs
'         outa[16..23]~

'     XB.rxFlush
  
PUB ChangeTransmitAddress

  outa[p_Red]~~                                         ' Turn on Red LED during the reprogramming
  DL_Addr := ina[p_IdHigh..p_IdLow]                     ' Read the RoBo Remote ID pins and assign them to DL_Adder
  XB.AT_Init                                            ' Fast AT updates
  XB.AT_ConfigVal(string("ATDL"), DL_Addr)              ' Assign Destination ID address
  outa[p_Red]~                                          ' Turn off the LED after the XBee is reprogrammed

PUB Diagnostics
  vp.config(string("var:leftppulse,rightpulse,PulseMin,PulseMax,TurnAdjust,dataIn,DL_Addr,Y"))
  vp.config(string("start:analog"))
  vp.config(string("dso:view=[leftppulse,rightpulse],timescale=10ms"))
  vp.config(string("lsa:view=rightpulse,timescale=10ms"))
   
  vp.share(@LeftPulse,@y)